"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cheerio_1 = __importDefault(require("cheerio"));
const webpack_config_1 = require("../templates/webpack.config");
const PLUGIN_NAME = 'InjectCommonChunksPlugin';
const INDEX = 'index.html';
const CHUNK_PREFIX = 'common';
const APP_JS = '[src="./build/dev/app.js"]';
const IS_COMMON_CHUNK_ATTRIBUTE = 'data-is-common-chunk';
function chunkToPath(chunk) {
    const fileName = chunk.files[0];
    return fileName.startsWith('./') ? fileName : `./${fileName}`;
}
class InjectCommonChunksPlugin {
    constructor({ live = false, watch = false } = { live: false, watch: false }) {
        this.live = live;
        this.watch = watch;
    }
    apply(compiler) {
        compiler.hooks.emit.tapPromise(PLUGIN_NAME, async (compilation) => {
            if (this.isTemplatesBuild(compilation))
                return;
            const $ = this.parseHtml(compilation);
            if ($) {
                this.cleanHtmlFromChunks($);
                const chunks = this.getCommonChunks(compilation);
                if (!chunks.length)
                    return;
                const chunkPaths = chunks.map(chunkToPath);
                this.appendChunksToHtml(compilation, $, chunkPaths);
                this.updateAsset(compilation, INDEX, $.html({ normalizeWhitespace: true }));
                this.updateAsset(compilation, 'build/dev/common-chunks-list.js', `window.__commonChunksList = [${chunkPaths.map(chunkPath => `'${chunkPath}'`).join(',')}]`);
            }
        });
    }
    isTemplatesBuild(compilation) {
        return Boolean(compilation.assets[webpack_config_1.filename]);
    }
    cleanHtmlFromChunks($) {
        $(`[${IS_COMMON_CHUNK_ATTRIBUTE}]`).remove();
    }
    getCommonChunks(compilation) {
        return compilation.chunks.filter((chunk) => chunk.name.startsWith(CHUNK_PREFIX));
    }
    appendChunksToHtml(compilation, $, chunks) {
        const prependToElement = $(APP_JS);
        if (!prependToElement) {
            const error = `${PLUGIN_NAME}: there is no app.js script in index.html`;
            compilation.errors.push(error);
            throw new Error(error);
        }
        chunks.forEach(pathToChunk => {
            prependToElement.before(`<script src="${pathToChunk}" ${IS_COMMON_CHUNK_ATTRIBUTE}></script>`);
        });
    }
    parseHtml(compilation) {
        const indexHtmlAsset = compilation.assets[INDEX];
        if (!indexHtmlAsset) {
            if (this.live || this.watch)
                return null;
            const errorMessage = `${PLUGIN_NAME}: there are no html file to inject chunks`;
            compilation.errors.push(errorMessage);
            throw new Error(errorMessage);
        }
        const htmlContent = compilation.assets[INDEX].source();
        const $ = cheerio_1.default.load(htmlContent, {
            decodeEntities: false,
            _useHtmlParser2: true,
            lowerCaseTags: false,
        });
        return $;
    }
    updateAsset(compilation, name, content) {
        compilation.assets[name] = {
            source() {
                return content;
            },
            size() {
                return this.source().length;
            }
        };
    }
}
exports.default = InjectCommonChunksPlugin;
//# sourceMappingURL=InjectCommonChunksPlugin.js.map